--
-- Reference design - Rotary encoder and simple outps on Spartan-3E Starter Kit (Revision C)
--
-- Ken Chapman - Xilinx Ltd - November 2005
-- Revised 20th February 2006
-- 
-- Kechagias Apostolos - June 2012
-- Added bits to pattern and debouncer
--
-- This design demonstrates how to interface to the rotary encoder and simple outps.
--    At the start, only one outp is on. 
--    Turning the rotary encoder to the left or right will cause
--    the outp which is on to appear to also move in the corresponding direction.
--    Pressing the rotary encoder will invert all outps so that only one is off.
--
-- The design also uses the 50MHz oscillator provided on the board.
--
-- Instructional value
--   Basic VHDL including definition of outputs and outputs.
--   UCF (User Constraints File) constraints to define pin assignments to match board.
--   UCF constraints to apply pull-up and pull-down resistors to output pins.
--   Detecting rotary movement.
--   Synchronous design.
--
------------------------------------------------------------------------------------
--
-- NOTICE:
--
-- Copyright Xilinx, Inc. 2006.   This code may be contain portions patented by other 
-- third parties.  By providing this core as one possible implementation of a standard,
-- Xilinx is making no representation that the provided implementation of this standard 
-- is free from any claims of infringement by any third party.  Xilinx expressly 
-- disclaims any warranty with respect to the adequacy of the implementation, including 
-- but not limited to any warranty or representation that the implementation is free 
-- from claims of any third party.  Furthermore, Xilinx is providing this core as a 
-- courtesy to you and suggests that you contact all third parties to obtain the 
-- necessary rights to use this implementation.
--
------------------------------------------------------------------------------------
--
-- Library declarations
--
-- Standard IEEE libraries
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--
------------------------------------------------------------------------------------
--
--
entity rotary_cntrl is
port (
	outp : out std_logic_vector(0 to 17);
	toggle_btn : in std_logic;
	input : inout std_logic_vector(0 to 17);
	rotary_a : in std_logic;
	rotary_b : in std_logic;
	clk : in std_logic);
end rotary_cntrl;
--
------------------------------------------------------------------------------------
--
-- Start of test architecture
--
architecture Behavioral of rotary_cntrl is
--
------------------------------------------------------------------------------------
--
-- Signals used to interface to rotary encoder
--
signal      rotary_a_in : std_logic;
signal      rotary_b_in : std_logic;
signal        rotary_in : std_logic_vector(1 downto 0);
signal        rotary_q1 : std_logic;
signal        rotary_q2 : std_logic;
signal  delay_rotary_q1 : std_logic;
signal     rotary_event : std_logic;
signal      rotary_left : std_logic;
--
-- Signals used to drive outps
--
signal      outp_pattern : std_logic_vector(0 to 17):= "000000000000000001"; --initial value puts one outp on near the middle.
signal        outp_drive : std_logic_vector(0 to 17);

-- Debounce signals for toggle_btn

signal Q1, Q2, Q3, Q_OUT : std_logic;
--
------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--
-- Start of circuit description
--
begin
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- Interface to rotary encoder.
  -- Detection of movement and direction.
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  -- The rotary switch contacts are filtered using their offset (one-hot) style to  
  -- clean them. Circuit concept by Peter Alfke.
  -- Note that the clock rate is fast compared with the switch rate.

  rotary_filter: process(clk)
  begin
    if rising_edge(clk) then

      --Synchronise outputs to clock domain using flip-flops in output/output blocks.
      rotary_a_in <= rotary_a;
      rotary_b_in <= rotary_b;

      --concatinate rotary output signals to form vector for case construct.
      rotary_in <= rotary_b_in & rotary_a_in;

      case rotary_in is

        when "00" => rotary_q1 <= '0';         
                     rotary_q2 <= rotary_q2;
 
        when "01" => rotary_q1 <= rotary_q1;
                     rotary_q2 <= '0';

        when "10" => rotary_q1 <= rotary_q1;
                     rotary_q2 <= '1';

        when "11" => rotary_q1 <= '1';
                     rotary_q2 <= rotary_q2; 

        when others => rotary_q1 <= rotary_q1; 
                       rotary_q2 <= rotary_q2; 
      end case;

    end if;
  end process rotary_filter;
  --
  -- The rising edges of 'rotary_q1' indicate that a rotation has occurred and the 
  -- state of 'rotary_q2' at that time will indicate the direction. 
  --
  direction: process(clk)
  begin
    if rising_edge(clk) then

      delay_rotary_q1 <= rotary_q1;
      if rotary_q1='1' and delay_rotary_q1='0' then
        rotary_event <= '1';
        rotary_left <= rotary_q2;
       else
        rotary_event <= '0';
        rotary_left <= rotary_left;
      end if;

    end if;
  end process direction;
  --
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- outp control.
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  -- The outp pattern is provided by a shift register which has been initialised in the signal 
  -- definition to have only one bit active (High). 
  --
  outp_display: process(clk)
  begin
    if rising_edge(clk) then

      if rotary_event='1' then
        if rotary_left='1' then 
          outp_pattern <= outp_pattern(1 to 17) & outp_pattern(0); --rotate outps to left
         else
          outp_pattern <= outp_pattern(17) & outp_pattern(0 to 16); --rotate outps to right
        end if;
      end if;

		
     --
     -- Pressing the rotary encoder will cause all outp drive bits to be inverted.
     --

      
      outp_drive <= outp_pattern; 

      --Ouput outp drive to the pins making use of the output flip-flops in output/output blocks.
      outp <= outp_drive; 

    end if;
  end process outp_display;
  
  
  -- Improves it a bit but still bounces
  debouncer:process(clk)
	begin
		if rising_edge(clk) then
				Q1 <= toggle_btn;
				Q2 <= Q1;
				Q3 <= Q2;
			end if;
	end process;
  
  Q_OUT <= Q1 and Q2 and (not Q3);
  
  process(clk)
	begin
		if rising_edge(clk) then
			if Q_OUT ='1' then
				if(outp_pattern =     "100000000000000000") then 
					input(0) <= not input(0);
					
				elsif (outp_pattern = "010000000000000000") then
					 input(1) <= not input(1);
					 
				elsif (outp_pattern = "001000000000000000") then
					input(2) <= not input(2);
				  
				elsif (outp_pattern = "000100000000000000") then
					input(3) <= not input(3);
					
				elsif (outp_pattern = "000010000000000000") then
					input(4)<= not input(4);
			  
				elsif (outp_pattern = "000001000000000000") then
					input(5) <= not input(5);
				  
				elsif (outp_pattern = "000000100000000000") then
					input(6) <= not input(6);
				  
				elsif (outp_pattern = "000000010000000000") then
					input(7) <= not input(7);
				 
				elsif (outp_pattern = "000000001000000000") then
					input(8) <= not input(8);
				  
				elsif (outp_pattern = "000000000100000000") then
					input(9) <= not input(9);
				  
				elsif (outp_pattern = "000000000010000000") then
					input(10) <= not input(10);
				 
				elsif (outp_pattern = "000000000001000000") then
					input(11) <= not input(11);
				  
				elsif (outp_pattern = "000000000000100000") then
					input(12) <= not input(12);
				
				elsif (outp_pattern = "000000000000010000") then
					input(13) <= not input(13);
					
				elsif (outp_pattern = "000000000000001000") then
					input(14) <= not input(14);
					 
				elsif (outp_pattern = "000000000000000100") then
					input(15) <= not input(15);
					 
				elsif (outp_pattern = "000000000000000010") then
					input(16) <= not input(16);
					
				elsif (outp_pattern = "000000000000000001") then
					input(17) <= not input(17);
				
				end if;
			end if;
		end if;
	end process;
  --
  --
end Behavioral;

------------------------------------------------------------------------------------------------------------------------------------
--
-- END OF FILE rotary_cntrl.vhd
--
------------------------------------------------------------------------------------------------------------------------------------

